# -*- coding: utf-8 -*-

"""Utilities to work with :mod:`requests`."""

from __future__ import annotations

__all__ = ['generate_response', 'raise_http']

import json
import typing

import requests


REASONS: typing.Final[typing.Mapping[int, str]] = {
    100: 'Continue',
    101: 'Switching protocols',
    102: 'Processing',
    103: 'Early Hints',
    200: 'OK',
    201: 'Created',
    202: 'Accepted',
    203: 'Non-Authoritative Information',
    204: 'No Content',
    205: 'Reset Content',
    206: 'Partial Content',
    207: 'Multi-Status',
    208: 'Already Reported',
    226: 'IM Used',
    300: 'Multiple Choices',
    301: 'Moved Permanently',
    302: 'Found (Previously "Moved Temporarily")',
    303: 'See Other',
    304: 'Not Modified',
    305: 'Use Proxy',
    306: 'Switch Proxy',
    307: 'Temporary Redirect',
    308: 'Permanent Redirect',
    400: 'Bad Request',
    401: 'Unauthorized',
    402: 'Payment Required',
    403: 'Forbidden',
    404: 'Not Found',
    405: 'Method Not Allowed',
    406: 'Not Acceptable',
    407: 'Proxy Authentication Required',
    408: 'Request Timeout',
    409: 'Conflict',
    410: 'Gone',
    411: 'Length Required',
    412: 'Precondition Failed',
    413: 'Payload Too Large',
    414: 'URI Too Long',
    415: 'Unsupported Media Type',
    416: 'Range Not Satisfiable',
    417: 'Expectation Failed',
    418: 'I\'m a Teapot',
    421: 'Misdirected Request',
    422: 'Unprocessable Entity',
    423: 'Locked',
    424: 'Failed Dependency',
    425: 'Too Early',
    426: 'Upgrade Required',
    428: 'Precondition Required',
    429: 'Too Many Requests',
    431: 'Request Header Fields Too Large',
    451: 'Unavailable For Legal Reasons',
    500: 'Internal Server Error',
    501: 'Not Implemented',
    502: 'Bad Gateway',
    503: 'Service Unavailable',
    504: 'Gateway Timeout',
    505: 'HTTP Version Not Supported',
    506: 'Variant Also Negotiates',
    507: 'Insufficient Storage',
    508: 'Loop Detected',
    510: 'Not Extended',
    511: 'Network Authentication Required',
}


def generate_response(  # pylint: disable=too-many-arguments
        *,
        method: str = 'GET',
        url: str = '',
        status_code: int = 200,
        content: typing.Any = b'',
        encoding: str = 'utf-8',
        request_extras: typing.Optional[typing.Mapping[str, typing.Any]] = None,
) -> requests.Response:
    """Generate fake :class:`~requests.models.Response` with requested content."""
    result: requests.Response = requests.Response()

    if isinstance(content, str):
        content = content.encode(encoding)
    elif not isinstance(content, bytes):
        content = json.dumps(content).encode(encoding)

    result._content = content  # pylint: disable=protected-access
    result._content_consumed = True  # type: ignore  # pylint: disable=protected-access
    result.status_code = status_code
    result.url = url
    result.encoding = encoding
    result.reason = REASONS.get(status_code, '')

    if request_extras is None:
        request_extras = {}
    result.request = requests.PreparedRequest()
    result.request.prepare(method=method, url=url, **request_extras)

    return result


def raise_connection() -> typing.NoReturn:
    """Raise a :class:`requests.ConnectionError` error."""
    raise requests.ConnectionError('Server unavailable')


def raise_http(*, status_code: int = 200, **kwargs: typing.Any) -> typing.NoReturn:
    """
    Raise a :class:`requests.HTTPError` error.

    See :func:`~generate_response` for additional parameters. Response instance will be included in the error.
    """
    raise requests.HTTPError(f'HTTP {status_code}', response=generate_response(status_code=status_code, **kwargs))
